#version 150

#moj_import <fog.glsl>
#moj_import <hue_shift.glsl>

uniform sampler2D Sampler0;

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;

in float vertexDistance;
in vec4 vertexColor;
in vec2 texCoord0;
in vec2 texCoord1;
in vec4 normal;
in vec4 tintColour;

out vec4 fragColor;

/**
 * This changes the colour of XP orbs.
 *
 * I stole some of the code from this funky individual (planet minecraft, point and laugh):
 * https://www.planetminecraft.com/texture-pack/rainbow-xp-vanilla/
 *
 * Simply hue shifting the XP colours worked sufficiently well for me, but if I wanted
 * better control over the resulting colours I'd have to put more effort into this.
 * The resulting colour pulses between purple and dark pink.
 */

void main() {
    vec4 color = texture(Sampler0, texCoord0) * vertexColor * ColorModulator;
    if (color.a < 0.1) {
        discard;
    }

    // detect xp orbs using their tint colour
    vec4 col255 = tintColour * 255;
    if(
        0 <= col255.r && col255.r <= 255 &&
        253 <= col255.g && col255.g <= 255 &&
        0 <= col255.b && col255.b <= 51
    ) {
        color = vec4(hueShift(color.rgb, 160), color.a);
    }

    fragColor = linear_fog(color, vertexDistance, FogStart, FogEnd, FogColor);
}
